# Generated by roxygen2: do not edit by hand

S3method(coef,susie)
S3method(predict,susie)
S3method(print,summary.susie)
S3method(summary,susie)
export(coef.susie)
export(compute_ss)
export(compute_suff_stat)
export(estimate_s_rss)
export(get_cs_correlation)
export(kriging_rss)
export(predict.susie)
export(print.summary.susie)
export(summary.susie)
export(susie)
export(susie_auto)
export(susie_get_cs)
export(susie_get_lfsr)
export(susie_get_niter)
export(susie_get_objective)
export(susie_get_pip)
export(susie_get_posterior_mean)
export(susie_get_posterior_samples)
export(susie_get_posterior_sd)
export(susie_get_prior_variance)
export(susie_get_residual_variance)
export(susie_init_coef)
export(susie_plot)
export(susie_plot_changepoint)
export(susie_plot_iteration)
export(susie_rss)
export(susie_suff_stat)
export(susie_trendfilter)
export(univariate_regression)
importFrom(Matrix,colMeans)
importFrom(Matrix,colSums)
importFrom(Matrix,crossprod)
importFrom(Matrix,diag)
importFrom(Matrix,rowSums)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,summary)
importFrom(Matrix,t)
importFrom(Matrix,tcrossprod)
importFrom(crayon,combine_styles)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,annotate)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(matrixStats,colSds)
importFrom(methods,as)
importFrom(mixsqp,mixsqp)
importFrom(reshape,melt)
importFrom(stats,.lm.fit)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cov2cor)
importFrom(stats,dnorm)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pnorm)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,summary.lm)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(utils,head)
importFrom(utils,modifyList)
